@@ -421,17 +421,13 @@ meta_begin_modal_for_plugin (MetaScreen       *screen,
    * merge the two.
    */
   MetaDisplay *display = meta_screen_get_display (screen);
-  gboolean ok;
 
   if (is_modal (display) || display->grab_op != META_GRAB_OP_NONE)
     return FALSE;
 
-  if (meta_is_wayland_compositor ())
-    ok = TRUE;
-  else
-    ok = begin_modal_x11 (screen, plugin, options, timestamp);
-  if (!ok)
-    return FALSE;
+  if (!meta_is_wayland_compositor ())
+    if (!begin_modal_x11 (screen, plugin, options, timestamp))
+      return FALSE;
 
   display->grab_op = META_GRAB_OP_COMPOSITOR;
   display->grab_window = NULL;
@@ -439,6 +435,9 @@ meta_begin_modal_for_plugin (MetaScreen       *screen,
   display->grab_have_pointer = TRUE;
   display->grab_have_keyboard = TRUE;
 
+  if (meta_is_wayland_compositor ())
+    meta_display_sync_wayland_input_focus (display);
+
   return TRUE;
 }
 
@@ -452,17 +451,21 @@ meta_end_modal_for_plugin (MetaScreen     *screen,
 
   g_return_if_fail (is_modal (display));
 
-  if (!meta_is_wayland_compositor ())
-    {
-      XIUngrabDevice (xdpy, META_VIRTUAL_CORE_POINTER_ID, timestamp);
-      XIUngrabDevice (xdpy, META_VIRTUAL_CORE_KEYBOARD_ID, timestamp);
-    }
-
   display->grab_op = META_GRAB_OP_NONE;
   display->grab_window = NULL;
   display->grab_screen = NULL;
   display->grab_have_pointer = FALSE;
   display->grab_have_keyboard = FALSE;
+
+  if (meta_is_wayland_compositor ())
+    {
+      meta_display_sync_wayland_input_focus (display);
+    }
+  else
+    {
+      XIUngrabDevice (xdpy, META_VIRTUAL_CORE_POINTER_ID, timestamp);
+      XIUngrabDevice (xdpy, META_VIRTUAL_CORE_KEYBOARD_ID, timestamp);
+    }
 }
 
 static void
@@ -843,29 +846,6 @@ meta_compositor_window_surface_changed (MetaCompositor *compositor,
   meta_window_actor_update_surface (window_actor);
 }
 
-static gboolean
-grab_op_is_clicking (MetaGrabOp grab_op)
-{
-  switch (grab_op)
-    {
-    case META_GRAB_OP_CLICKING_MINIMIZE:
-    case META_GRAB_OP_CLICKING_MAXIMIZE:
-    case META_GRAB_OP_CLICKING_UNMAXIMIZE:
-    case META_GRAB_OP_CLICKING_DELETE:
-    case META_GRAB_OP_CLICKING_MENU:
-    case META_GRAB_OP_CLICKING_SHADE:
-    case META_GRAB_OP_CLICKING_UNSHADE:
-    case META_GRAB_OP_CLICKING_ABOVE:
-    case META_GRAB_OP_CLICKING_UNABOVE:
-    case META_GRAB_OP_CLICKING_STICK:
-    case META_GRAB_OP_CLICKING_UNSTICK:
-      return TRUE;
-
-    default:
-      return FALSE;
-    }
-}
-
 static gboolean
 event_is_passive_button_grab (MetaDisplay   *display,
                               XIDeviceEvent *device_event)
@@ -921,7 +901,7 @@ maybe_spoof_event_as_stage_event (MetaCompScreen *info,
           /* If this is a window frame, and we think GTK+ needs to handle the event,
              let GTK+ handle it without mangling */
           if (window && window->frame && device_event->event == window->frame->xwindow &&
-              (grab_op_is_clicking (display->grab_op) ||
+              (meta_grab_op_is_clicking (display->grab_op) ||
                (display->grab_op == META_GRAB_OP_NONE && !event_is_passive_button_grab (display, device_event))))
             break;
 
