@@ -20,7 +20,6 @@
 #include "frame.h"
 #include <meta/window.h>
 #include <meta/meta-shaped-texture.h>
-#include "xprops.h"
 
 #include "compositor-private.h"
 #include "meta-shaped-texture-private.h"
@@ -30,11 +29,12 @@
 #include "region-utils.h"
 #include "monitor-private.h"
 #include "meta-cullable.h"
-#include "meta-wayland-private.h"
 
 #include "meta-surface-actor.h"
 #include "meta-surface-actor-x11.h"
 
+#include "wayland/meta-wayland-surface.h"
+
 struct _MetaWindowActorPrivate
 {
   MetaWindow       *window;
@@ -714,7 +714,7 @@ meta_window_actor_has_shadow (MetaWindowActor *self)
   /* Leaving out shadows for maximized and fullscreen windows is an effeciency
    * win and also prevents the unsightly effect of the shadow of maximized
    * window appearing on an adjacent window */
-  if ((meta_window_get_maximized (priv->window) == (META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL)) ||
+  if ((meta_window_get_maximized (priv->window) == META_MAXIMIZE_BOTH) ||
       meta_window_is_fullscreen (priv->window))
     return FALSE;
 
@@ -968,9 +968,6 @@ start_simple_effect (MetaWindowActor *self,
   gint *counter = NULL;
   gboolean use_freeze_thaw = FALSE;
 
-  if (!info->plugin_mgr)
-    return FALSE;
-
   switch (event)
   {
   case META_PLUGIN_MINIMIZE:
@@ -1299,8 +1296,7 @@ meta_window_actor_maximize (MetaWindowActor    *self,
   self->priv->maximize_in_progress++;
   meta_window_actor_freeze (self);
 
-  if (!info->plugin_mgr ||
-      !meta_plugin_manager_event_maximize (info->plugin_mgr,
+  if (!meta_plugin_manager_event_maximize (info->plugin_mgr,
                                            self,
                                            META_PLUGIN_MAXIMIZE,
                                            new_rect->x, new_rect->y,
@@ -1328,8 +1324,7 @@ meta_window_actor_unmaximize (MetaWindowActor   *self,
   self->priv->unmaximize_in_progress++;
   meta_window_actor_freeze (self);
 
-  if (!info->plugin_mgr ||
-      !meta_plugin_manager_event_maximize (info->plugin_mgr,
+  if (!meta_plugin_manager_event_maximize (info->plugin_mgr,
                                            self,
                                            META_PLUGIN_UNMAXIMIZE,
                                            new_rect->x, new_rect->y,
