@@ -145,8 +145,6 @@ struct _MetaDisplay
    * multiple events with the same serial.
    */
   guint focused_by_us : 1;
-
-  guint static_gravity_works : 1;
   
   /*< private-ish >*/
   guint error_trap_synced_at_last_pop : 1;
@@ -456,6 +454,9 @@ int meta_resize_gravity_from_grab_op (MetaGrabOp op);
 gboolean meta_grab_op_is_moving   (MetaGrabOp op);
 gboolean meta_grab_op_is_resizing (MetaGrabOp op);
 gboolean meta_grab_op_is_mouse    (MetaGrabOp op);
+gboolean meta_grab_op_is_clicking (MetaGrabOp op);
+gboolean meta_grab_op_is_wayland  (MetaGrabOp op);
+gboolean meta_grab_op_is_keyboard (MetaGrabOp op);
 
 void meta_display_devirtualize_modifiers (MetaDisplay        *display,
                                           MetaVirtualModifier modifiers,
@@ -488,4 +489,16 @@ void meta_display_set_input_focus_xwindow (MetaDisplay *display,
                                            Window       window,
                                            guint32      timestamp);
 
+void meta_display_sync_wayland_input_focus (MetaDisplay *display);
+void meta_display_update_focus_window (MetaDisplay *display,
+                                       MetaWindow  *window,
+                                       Window       xwindow,
+                                       gulong       serial,
+                                       gboolean     focused_by_us);
+
+void meta_display_sanity_check_timestamps (MetaDisplay *display,
+                                           guint32      timestamp);
+gboolean meta_display_timestamp_too_old (MetaDisplay *display,
+                                         guint32     *timestamp);
+
 #endif
