@@ -50,10 +50,8 @@
 #include "display-private.h"
 #include <meta/errors.h>
 #include "ui.h"
-#include "session.h"
 #include <meta/prefs.h>
 #include <meta/compositor.h>
-#include "meta-wayland.h"
 
 #include <glib-object.h>
 #include <glib-unix.h>
@@ -79,6 +77,10 @@
 #include <girepository.h>
 #endif
 
+#include "x11/session.h"
+
+#include "wayland/meta-wayland.h"
+
 /*
  * The exit code we'll return to our parent process when we eventually die.
  */
@@ -190,6 +192,7 @@ static gboolean  opt_replace_wm;
 static gboolean  opt_disable_sm;
 static gboolean  opt_sync;
 static gboolean  opt_wayland;
+static gboolean  opt_display_server;
 
 static GOptionEntry meta_options[] = {
   {
@@ -233,6 +236,11 @@ static GOptionEntry meta_options[] = {
     N_("Run as a wayland compositor"),
     NULL
   },
+  {
+    "display-server", 0, 0, G_OPTION_ARG_NONE,
+    &opt_display_server,
+    N_("Run as a full display server, rather than nested")
+  },
   {NULL}
 };
 
@@ -401,8 +409,7 @@ meta_init (void)
   if (g_getenv ("MUTTER_DEBUG"))
     meta_set_debugging (TRUE);
 
-  /* We consider running from mutter-launch equivalent to running from bare metal. */
-  if (getenv ("WESTON_LAUNCHER_SOCK"))
+  if (opt_display_server)
     clutter_set_windowing_backend (CLUTTER_WINDOWING_EGL);
 
   meta_set_is_wayland_compositor (opt_wayland);
@@ -497,6 +504,32 @@ meta_register_with_session (void)
   g_free (opt_client_id);
 }
 
+/**
+ * meta_activate_session:
+ *
+ * Tells mutter to activate the session. When mutter is a
+ * Wayland compositor, this tells logind to switch over to
+ * the new session.
+ */
+gboolean
+meta_activate_session (void)
+{
+  if (meta_is_wayland_compositor ())
+    {
+      MetaWaylandCompositor *compositor = meta_wayland_compositor_get_default ();
+      GError *error = NULL;
+
+      if (!meta_wayland_compositor_activate_session (compositor, &error))
+        {
+          g_warning ("Could not activate session: %s\n", error->message);
+          g_error_free (error);
+          return FALSE;
+        }
+    }
+
+  return TRUE;
+}
+
 /**
  * meta_run: (skip)
  *
