@@ -65,13 +65,28 @@ typedef enum {
 
 #define NUMBER_OF_QUEUES 3
 
-
 typedef enum {
   _NET_WM_BYPASS_COMPOSITOR_HINT_AUTO = 0,
   _NET_WM_BYPASS_COMPOSITOR_HINT_ON = 1,
   _NET_WM_BYPASS_COMPOSITOR_HINT_OFF = 2,
 } MetaBypassCompositorHintValue;
 
+typedef enum
+{
+  META_IS_CONFIGURE_REQUEST = 1 << 0,
+  META_DO_GRAVITY_ADJUST    = 1 << 1,
+  META_IS_USER_ACTION       = 1 << 2,
+  META_IS_MOVE_ACTION       = 1 << 3,
+  META_IS_RESIZE_ACTION     = 1 << 4,
+  META_IS_WAYLAND_RESIZE    = 1 << 5
+} MetaMoveResizeFlags;
+
+typedef enum
+{
+  META_MOVE_RESIZE_RESULT_MOVED               = 1 << 0,
+  META_MOVE_RESIZE_RESULT_RESIZED             = 1 << 1,
+  META_MOVE_RESIZE_RESULT_FRAME_SHAPE_CHANGED = 1 << 2,
+} MetaMoveResizeResultFlags;
 
 struct _MetaWindow
 {
@@ -88,7 +103,6 @@ struct _MetaWindow
   MetaFrame *frame;
   int depth;
   Visual *xvisual;
-  Colormap colormap;
   char *desc; /* used in debug spew */
   char *title;
 
@@ -454,6 +468,21 @@ struct _MetaWindowClass
 {
   GObjectClass parent_class;
 
+  void (*manage)                 (MetaWindow *window);
+  void (*unmanage)               (MetaWindow *window);
+  void (*ping)                   (MetaWindow *window,
+                                  guint32     serial);
+  void (*delete)                 (MetaWindow *window,
+                                  guint32     timestamp);
+  void (*kill)                   (MetaWindow *window);
+  void (*focus)                  (MetaWindow *window,
+                                  guint32     timestamp);
+  void (*move_resize_internal)   (MetaWindow                *window,
+                                  int                        gravity,
+                                  MetaRectangle              requested_rect,
+                                  MetaRectangle              constrained_rect,
+                                  MetaMoveResizeFlags        flags,
+                                  MetaMoveResizeResultFlags *result);
   void (*get_default_skip_hints) (MetaWindow *window,
                                   gboolean   *skip_taskbar_out,
                                   gboolean   *skip_pager_out);
@@ -582,12 +611,6 @@ void        meta_window_get_geometry         (MetaWindow  *window,
 
 void        meta_window_update_unfocused_button_grabs (MetaWindow *window);
 
-/* Sends a client message */
-void meta_window_send_icccm_message (MetaWindow *window,
-                                     Atom        atom,
-                                     guint32     timestamp);
-
-
 void     meta_window_move_resize_request(MetaWindow *window,
                                          guint       value_mask,
                                          int         gravity,
@@ -730,4 +753,9 @@ void meta_window_update_monitor (MetaWindow *window);
 void meta_window_set_urgent (MetaWindow *window,
                              gboolean    urgent);
 
+void meta_window_update_resize (MetaWindow *window,
+                                gboolean    snap,
+                                int x, int y,
+                                gboolean force);
+
 #endif
